# Generated by Django 5.0.4 on 2024-04-05 20:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_quotes", "0011_alter_source_text_model_alter_sourcegroup_text_model"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="sourcemarkovmodel",
            name="source",
        ),
        migrations.DeleteModel(
            name="GroupMarkovModel",
        ),
        migrations.DeleteModel(
            name="SourceMarkovModel",
        ),
    ]
